﻿using System;
using System.ComponentModel;
using System.Linq.Expressions;

namespace XGrid
{
    public class MainViewModel : INotifyPropertyChanged
    {
        private DataSource _dataSource;
        private bool _isBusy;

        public MainViewModel()
        {
            BackgroundWorker bw = new BackgroundWorker();

            bw.DoWork += (o, e) =>
            {
                e.Result = new DataSource(50, 4);
            };

            bw.RunWorkerCompleted += (o, e) =>
            {
                if (e.Error == null)
                {
                    this.DataSource = (DataSource)e.Result;
                    this.IsBusy = false;
                }
            };

            bw.RunWorkerAsync();
            this.IsBusy = true;
        }

        public bool IsBusy
        {
            get
            {
                return this._isBusy;
            }

            set
            {
                this._isBusy = value;
                this.RaisePropertyChanged(() => this.IsBusy);
            }
        }

        public DataSource DataSource
        {
            get
            {
                return this._dataSource;
            }

            set
            {
                this._dataSource = value;
                this.RaisePropertyChanged(() => this.DataSource);
            }
        }

        #region Notify

        public event PropertyChangedEventHandler PropertyChanged;

        protected static string GetPropertyName<T>(Expression<Func<T>> action)
        {
            var expression = (MemberExpression)action.Body;
            return expression.Member.Name;
        }

        protected virtual void RaisePropertyChanged<T>(Expression<Func<T>> action)
        {
            string propertyName = GetPropertyName(action);

            this.OnPropertyChanged(propertyName);
        }

        private void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion // Notify
    }
}
